// LED Options Parser
// p = Pin Assignment. Example atlp113 sets LED1 Pin to 13. Pin number may not be higher than available pins
//           New Pin will be checked for collision with other LED Pin and Button Pin.
//     atlps Shows Pin Assingment
//     atlpS Saves Pin Assignment to EEPROM
//     atlpr resets Pins to default if no Collision with key pins
// t = LED Timing-Modifier. atlt110000 sets the Modifier 1 to 10000 (default), atlts Shows Timings, atltS Saves Timings to EEPROM, atltr restores Default Timings
// m = LED Mode. atlm<decimal value> sets the Mode, atlms shows mode, atlmS Saves Mode to EEPROM, atlmr restores Default LED Mode
// o = LED Option. atlo<4 hex digits> sets the options. atlor restores Default Options
void parseled(char *c, unsigned int num) {
  unsigned int value;
  int n;
  bool x;
  switch (c[0]) {
    case 'p':
      if (num >= 3) {
        switch (c[1]) {
          case '1':
          case '2':
            value = (unsigned int) mj_convert_dec_long(c + 2);
            Serial.print(value);
            Serial.print(F("\r\n"));
            // Check collision with other Pins already used (Key Inputs)
            x = true;
            for (n = 0; n < 4; n++) {  // Avoid collision with our 4 Key-Pins
              if (value == mj_keypin[n]) {
                x = false;
              }
            }
#ifndef LED_BUILTIN_RX   // For example Arduino Uno - if we'd use the RX/TX Pins for LED here we kill the serial communication. Leonardo works with 0/1
            if (value == 0 || value == 1) {
              x = false;
            }
#endif
            if (c[1] == '1') {
              if (value == mj_ledpin2) {
                x = false;
              }
            } else {
              if (value == mj_ledpin1) {
                x = false;
              }
            }
            if (x == true && value < NUM_DIGITAL_PINS ) {
              pinMode(value, OUTPUT);
              if (c[1] == '1') {
                digitalWrite(mj_ledpin1, 0);
                mj_ledpin1 = value;
              } else {
                digitalWrite(mj_ledpin2, 0);
                mj_ledpin2 = value;
              }
            }
            break;
        }
      } else if (num == 2) {
        switch (c[1]) {
          case 'r': // Restore LED Default Pins
            if ((MJ_DEFAULT_LEDPIN_1 != mj_keypin[0]) && (MJ_DEFAULT_LEDPIN_1 != mj_keypin[1]) && (MJ_DEFAULT_LEDPIN_1 != mj_keypin[2]) && (MJ_DEFAULT_LEDPIN_1 != mj_keypin[3])) {
              mj_ledpin1 = MJ_DEFAULT_LEDPIN_1;
            }
            if ((MJ_DEFAULT_LEDPIN_2 != mj_keypin[0]) && (MJ_DEFAULT_LEDPIN_2 != mj_keypin[1]) && (MJ_DEFAULT_LEDPIN_2 != mj_keypin[2]) && (MJ_DEFAULT_LEDPIN_2 != mj_keypin[3])) {
              mj_ledpin2 = MJ_DEFAULT_LEDPIN_2;
            }
            pinMode(mj_ledpin1, OUTPUT);
            pinMode(mj_ledpin2, OUTPUT);
            [[fallthrough]];
          case 's': // Show serial Pin Assignment
            Serial.print(F("LEDPIN1: "));
            Serial.print(mj_ledpin1);
            Serial.print(F(" LEDPIN2: "));
            Serial.print(mj_ledpin2);
            Serial.print(F("\r\n"));
            break;
          case 'S': // Save Serial Pin Assignment to EEPROM
            mj_eeprom_write_int(MJ_EEPROM_LEDPIN_1, mj_ledpin1); // Save Pin 1
            mj_eeprom_update_value(MJ_EEPROM_LEDPIN_2, 2, mj_ledpin2); // Save Pin 2 plus recalculate + Update CRC
            break;
          default:
            break;
        }
      }
      break;
    case 't': // LED Timing Modifiers
      if (num >= 2) {
        switch (c[1]) {
          case '1':
          case '2':
          case '3':
          case '4':
            if (num > 2) {
              value = (unsigned int) mj_convert_dec_long(c + 2);
              n = c[1] - '1';
              mj_ledtiming[n] = value;
              mj_ledtiming_changed = true; // Force Recalculation of LED Timings
            }
            break;
          case 'r': // Restore Default Timings
            mj_ledtiming[0] = MJ_DEFAULT_LEDTIMING_1;
            mj_ledtiming[1] = MJ_DEFAULT_LEDTIMING_2;
            mj_ledtiming[2] = MJ_DEFAULT_LEDTIMING_3;
            mj_ledtiming[3] = MJ_DEFAULT_LEDTIMING_4;
            mj_ledtiming_changed = true;
            [[fallthrough]];
          case 's':
            Serial.print(F("LED Timing"));
            for (n = 0; n < 4; n++) {
              Serial.print(n + 1);
              Serial.print(F(":"));
              Serial.print(mj_ledtiming[n]);
              Serial.write(' ');
            }
            Serial.print(F("\r\n"));
            break;
          case 'S': // Save LED Timings to EEPROM
            mj_eeprom_write_int(MJ_EEPROM_LEDTIMING_1, mj_ledtiming[0]);
            mj_eeprom_write_int(MJ_EEPROM_LEDTIMING_2, mj_ledtiming[1]);
            mj_eeprom_write_int(MJ_EEPROM_LEDTIMING_3, mj_ledtiming[2]);
            mj_eeprom_update_value(MJ_EEPROM_LEDTIMING_4, 2, mj_ledtiming[3]);
            break;
          default:
            break;
        }
      }
      break;
    case 'm': // LED Mode
      if (num >= 2) {
        switch (c[1]) {
          case '0':
          case '1':
          case '2':
          case '3':
          case '4':
          case '5':
          case '6':
          case '7':
          case '8':
          case '9':
            value = (unsigned int) mj_convert_dec_long(c + 1);
            mj_ledmode = value;
            mj_ledtiming_changed = true; // If LED Mode changes then recalculate Timings in LED Process
            break;
          case 'r':
            mj_ledmode = MJ_DEFAULT_LEDMODE;
            mj_ledtiming_changed = true; // If LED Mode changes then recalculate Timings in LED Process
            [[fallthrough]];
          case 's':
            Serial.print(F("LED Mode: "));
            Serial.print(mj_ledmode);
            Serial.print(F("\r\n"));
            break;
          case 'S':
            mj_eeprom_update_value(MJ_EEPROM_LEDMODE, 2, mj_ledmode);
            break;
          default:
            break;
        }
      }
      break;
    case 'o': // LED Options
      if (num >= 2) {
        switch (c[1]) {
          case '0':
          case '1':
          case '2':
          case '3':
          case '4':
          case '5':
          case '6':
          case '7':
          case '8':
          case '9':
          case 'a':
          case 'b':
          case 'c':
          case 'd':
          case 'e':
          case 'f':
          case 'A':
          case 'B':
          case 'C':
          case 'D':
          case 'E':
          case 'F':
            value = (unsigned int) mj_convert_hex_long(c + 1);
            mj_ledoption = value;
            mj_ledtiming_changed = true; // If LED Option changes then recalculate Timings in LED Process
            break;
          case 'r':
            mj_ledoption = MJ_DEFAULT_LEDOPTION;
            mj_ledtiming_changed = true; // If LED Option changes then recalculate Timings in LED Process
            [[fallthrough]];
          case 's':
            Serial.print(F("LED Option: 0x"));
            Serial.print(mj_ledoption, HEX);
            Serial.print(F("\r\n"));
            break;
          case 'S':
            mj_eeprom_update_value(MJ_EEPROM_LEDOPTION, 2, mj_ledoption);
            break;
          default:
            break;
        }
      }
      break;
  }
}

// Commands ate...
// d = dump eeprom contents
// c = Calc CRC and show calculated and stored CRC
// C = Calc CRC and update stored CDC
// i,I,z = initialize EEPROM Contents - need to be given 3 times - i just rewrites known content, I wipes all, z only zeroes whole eeprom (0xff) but does not load defaults
// w,W = Write value b/c = byte, i = int, l = long to given address (3 hex digits) with value (1-8 digits). byte = 2, int = 4, long = 1-8 digits W with update CRC, w without update CRC
// r,R = Read value from address (same as w/W but without value) and show

void parseeeprom(char *c, unsigned int num) {
  unsigned long crc_calc, crc_stored;
  static int i = 2;
  unsigned long v;
  int t = 0, a = 0;
  switch (c[0]) {
    case 'i':
    case 'I':
    case 'z':
      ;
      break;
    default:
      i = 2;
      break;
  }
  switch (c[0]) {
    case 'd':
      mj_eeprom_dump_serial();
      break;
    case 'c':
      crc_calc = mj_eeprom_calc_crc(MJ_EEPROM_CRCAREA_START, MJ_EEPROM_CRCAREA_END);
      crc_stored = mj_eeprom_read_long(MJ_EEPROM_CRC);
      Serial.print(F("EEPROM CRC "));
      Serial.write((crc_calc == crc_stored) ? "valid" : "invalid");
      Serial.print(F(" stored: "));
      Serial.print(crc_stored);
      Serial.print(F(" calculated: "));
      Serial.print(crc_calc);
      Serial.print(F("\r\n"));
      break;
    case 'C':
      crc_calc = mj_eeprom_calc_crc(MJ_EEPROM_CRCAREA_START, MJ_EEPROM_CRCAREA_END);
      mj_eeprom_write_long(MJ_EEPROM_CRC, crc_calc);
      break;
    case 'i':
    case 'I':
    case 'z':
      if (i > 0) {
        Serial.print(F("Please repeat command "));
        Serial.print((i == 2) ? "twice" : "once");
        Serial.print(F(" to wipe EEPROM\r\n"));
        i--;
      } else {
        i = 2;
        if (c[0] == 'I' || c[0] == 'z') {
          Serial.print(F("Wiping\r\n"));
          mj_eeprom_wipe();
        }
        if (c[0] == 'I' || c[0] == 'i') {
          Serial.print(F("Initializing defaults\r\n"));
          mj_eeprom_init_defaults();
          mj_eeprom_get_config();
        }
      }
      break;
    case 'w':
    case 'W':  // Write wants to have 5,7 or 11 Hex digits depending on modifier: atwb 5, atwi 7, atwl 11. first 3 digits address, remaining value.
      if (num >= 6) {
        a = mj_convert_hex_int('0', c[2], c[3], c[4]);
      }
      switch (c[1]) {
        case 'b':
        case 'c': // One char
          if (num == 7) {
            v = (unsigned long) mj_convert_hex_char(c[5], c[6]);
            t = 1;
          }
          break;
        case 'i': // int
          if (num == 9) {
            v = (unsigned long) mj_convert_hex_int(c[5], c[6], c[7], c[8]);
            t = 2;
          }
          break;
        case 'l': // long
          if (num >= 6) {
            v = mj_convert_hex_long(c + 5);
            t = 4;
          }
          break;
      }
      if (t > 0) {
        switch (t) {
          case 1:
            mj_eeprom_write_byte(a, (byte) v);
            break;
          case 2:
            mj_eeprom_write_int(a, (unsigned int) v);
            break;
          case 4:
            mj_eeprom_write_long(a, v);
            break;
          default:
            break;
        }
        if (c[0] == 'W') {
          crc_calc = mj_eeprom_calc_crc(MJ_EEPROM_CRCAREA_START, MJ_EEPROM_CRCAREA_END);
          mj_eeprom_write_long(MJ_EEPROM_CRC, crc_calc);
        }
      }
      break;
    case 'r':
    case 'R':  // Read wants to have 3 hex digits address.
      if (num == 5) {
        a = mj_convert_hex_int('0', c[2], c[3], c[4]);
      }
      switch (c[1]) {
        case 'b':
        case 'c': // One char
          v = (unsigned long) mj_eeprom_read_byte(a);
          t = 1;
          break;
        case 'i': // int
          v = (unsigned long) mj_eeprom_read_int(a);
          t = 2;
          break;
        case 'l': // long
          v = mj_eeprom_read_long(a);
          t = 4;
          break;
        default:
          break;
      }
      if (t > 0) {
        Serial.print(F("Addr: "));
        Serial.print(a, HEX);
        Serial.print(F(" value: "));
        Serial.print(v, HEX);
        Serial.print(F("\r\n"));
      }
      break;
  }
}

// Command att...
// atts - print eeprom content to serial
// attk - write string after k to keyboard
// attw - overwrite string in eeprom - if no string is given this will mark the string deleted
// attt - type string in eeprom
// attd<0-9><decimal> - configure typedelay <0-9> to <decimal> ms
// attds - show active typedelays
// attdr - reset typedelays to default
// attdS - save active typedelays to EEPROM
// attar - reset autotype delay to default
// attas - show autotype delay
// attat<decimal> - set autotype delay to <decimal> in ms
// attad - disable autotyping
// attae - enable autotyping
// attaS - Save autotype delay to EEPROM

void parsekeyboard(char *c, unsigned int num) {
  unsigned int i = 0;
  unsigned long value = 0;
  switch (c[0]) {
    case 'a':
      if (num > 2 && c[1] == 't') {
        mj_autotype_delay = mj_convert_dec_long(c + 2);
      } else if (num == 2) {
        switch (c[1]) {
          case 'r':
            mj_autotype_delay = MJ_DEFAULT_AUTOTYPE_DELAY;
            mj_autotype = MJ_DEFAULT_AUTOTYPE;
            [[fallthrough]];
          case 's':
            Serial.print(F("AutoType is "));
            Serial.print(mj_autotype > 0 ? "en" : "dis");
            Serial.print(F("abled with delay "));
            Serial.print(mj_autotype_delay);
            Serial.print(F("\r\n"));
            break;
          case 'S':
            mj_eeprom_write_byte(MJ_EEPROM_AUTOTYPE, mj_autotype);
            mj_eeprom_update_value(MJ_EEPROM_AUTOTYPE_DELAY, 4, mj_autotype_delay);
            break;
          case 'e':
            mj_autotype = true;
            break;
          case 'd':
            mj_autotype = false;
            break;
          default:
            break;
        }
      }
      break;
    case 's':
      for (i = 0; i < mj_usedbytes && i < MJ_EEPROM_CONTENT_1_SIZE; i++) {
        Serial.write((char) mj_eeprom_read_byte(i + MJ_EEPROM_CONTENT_1_START));
      }
      Serial.print(F("\r\n"));
      break;
    case 'k':
      putfifo_string(c + 1, 0);
      break;
    case 'w':
      for (i = 0; (c[i + 1] > 0) && ( i < (num - 1) ) && (i < MJ_EEPROM_CONTENT_1_SIZE); i++) {
        mj_eeprom_write_byte(MJ_EEPROM_CONTENT_1_START + i, c[i + 1]);
      }
      mj_eeprom_write_byte(MJ_EEPROM_CONTENT_1_START + i, '\0');
      mj_usedbytes[1] = i;
      mj_eeprom_update_value(MJ_EEPROM_CONTENT_1_USEDBYTES, 2, mj_usedbytes[1]);
      break;
    case 't':
      putfifo_string(c, 1);
      break;
    case 'd':
      if (num == 2) {
        switch (c[1]) {
          case 'r': // reset eeprom defaults;
            mj_keydelay[0] = MJ_DEFAULT_KEYDELAY_0;
            mj_keydelay[1] = MJ_DEFAULT_KEYDELAY_1;
            mj_keydelay[2] = MJ_DEFAULT_KEYDELAY_2;
            mj_keydelay[3] = MJ_DEFAULT_KEYDELAY_3;
            mj_keydelay[4] = MJ_DEFAULT_KEYDELAY_4;
            mj_keydelay[5] = MJ_DEFAULT_KEYDELAY_5;
            mj_keydelay[6] = MJ_DEFAULT_KEYDELAY_6;
            mj_keydelay[7] = MJ_DEFAULT_KEYDELAY_7;
            mj_keydelay[8] = MJ_DEFAULT_KEYDELAY_8;
            mj_keydelay[9] = MJ_DEFAULT_KEYDELAY_9;
            [[fallthrough]];
          case 's': // show delays
            Serial.print(F("KeyType Delay List"));
            for (i = 0; i < 10; i++) {
              Serial.write(' ');
              Serial.print(i);
              Serial.print(F(":"));
              Serial.print(mj_keydelay[i]);
            }
            Serial.print(F("\r\n"));
            break;
          case 'S':
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_0, mj_keydelay[0]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_1, mj_keydelay[1]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_2, mj_keydelay[2]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_3, mj_keydelay[3]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_4, mj_keydelay[4]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_5, mj_keydelay[5]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_6, mj_keydelay[6]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_7, mj_keydelay[7]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_8, mj_keydelay[8]);
            mj_eeprom_update_value(MJ_EEPROM_KEYDELAY_9, 4, mj_keydelay[9]);
            break;
        }
      } else if (num >= 3) {
        if (c[1] >= '0' && c[1] <= '9') {
          i = c[1] - '0';
          value = mj_convert_dec_long(c + 2);
          mj_keydelay[i] = value;
        }
      }
    default:
      break;
  }
}

// Command atb
// atb<1-4><decimal> set button 4 to pin <decimal> - may not be higher than available pins
// atbr - reset button config to default
// atbs - show active button config
// atbS - save active button config to EEPROM
void parsebuttons(char *c, unsigned int num) {
  unsigned int value;
  int n = 0, i = 0;
  bool x;
  if (num >= 1) {
    switch (c[0]) {
      case '1':
      case '2':
      case '3':
      case '4':
        value = (unsigned int) mj_convert_dec_long(c + 1);
        i = c[0] - '1';
        // Check collision with other Pins already used (Key Inputs)
        x = true;
        for (n = 0; n < 4; n++) {
          if ( n != i ) {
            if (value == mj_keypin[n]) {
              x = false;
            }
          }
        }
        if (value == mj_ledpin1) {
          x = false;
        }
        if (value == mj_ledpin2) {
          x = false;
        }
#ifndef LED_BUILTIN_RX   // For example Arduino Uno - if we'd use the RX/TX Pins for Keys here we kill the serial communication. Leonardo works with 0/1
        if (value == 0 || value == 1) {
          x = false;
        }
#endif
        if (x == true && value < NUM_DIGITAL_PINS ) {
          pinMode(value, INPUT_PULLUP);
          mj_keypin[i] = value;
        }
        break;
      case 'r': // Restore Key Default
        if ( (MJ_DEFAULT_KEYPIN_1 != mj_ledpin1) && (MJ_DEFAULT_KEYPIN_1 != mj_ledpin2 ) &&
             (MJ_DEFAULT_KEYPIN_2 != mj_ledpin1) && (MJ_DEFAULT_KEYPIN_2 != mj_ledpin2 ) &&
             (MJ_DEFAULT_KEYPIN_3 != mj_ledpin1) && (MJ_DEFAULT_KEYPIN_3 != mj_ledpin2 ) &&
             (MJ_DEFAULT_KEYPIN_4 != mj_ledpin1) && (MJ_DEFAULT_KEYPIN_4 != mj_ledpin2 ) ) {
          mj_keypin[0] = MJ_DEFAULT_KEYPIN_1;
          mj_keypin[1] = MJ_DEFAULT_KEYPIN_2;
          mj_keypin[2] = MJ_DEFAULT_KEYPIN_3;
          mj_keypin[3] = MJ_DEFAULT_KEYPIN_4;
        }
        pinMode(mj_keypin[0], INPUT_PULLUP);
        pinMode(mj_keypin[1], INPUT_PULLUP);
        pinMode(mj_keypin[2], INPUT_PULLUP);
        pinMode(mj_keypin[3], INPUT_PULLUP);
        [[fallthrough]];
      case 's': // Show serial Pin Assignment
        Serial.print(F("KEYPIN "));
        for (n = 0; n < 4; n++) {
          Serial.print(n + 1);
          Serial.write(':');
          Serial.print(mj_keypin[n]);
          Serial.write(' ');
        }
        Serial.print(F("\r\n"));
        break;
      case 'S': // Save Serial Pin Assignment to EEPROM
        mj_eeprom_write_int(MJ_EEPROM_KEYPIN_1, mj_keypin[0]); // Save Pin 1
        mj_eeprom_write_int(MJ_EEPROM_KEYPIN_2, mj_keypin[1]); // Save Pin 2
        mj_eeprom_write_int(MJ_EEPROM_KEYPIN_3, mj_keypin[2]); // Save Pin 3
        mj_eeprom_update_value(MJ_EEPROM_KEYPIN_4, 2, mj_keypin[3]); // Save Pin 4 plus recalculate + Update CRC
        break;
      default:
        break;
    }
  }
}

// Jiggler options Parser
// atje enable Jiggler
// atjd disable Jiggler
// atjs show Jiggler state
// atjS Save Jiggler state
// atjr recover default Jiggler State
// atjt1<dezimal> Set Minimum Wait-Time between 2 Jiggles to number (0-65535) in ms
// atjt2<dezimal> Set Maximum Wait-Time between 2 Jiggles to number (0-65535) in ms
// atjts Show Jiggler timings
// atjtS Save Jiggler timings to EEPROM
// atjtr recover Jiggler Default State
// atjp<decimal> Set Pixel Max Pixel Movement to number (0-255)
// atjps Show Max Pixel Movement
// atjpS Save Max Pixel Movement to EEPROM
// atjpr recover Max Pixel Movement Default
void parsejiggler(char *c, unsigned int num) {
  unsigned int value;
  switch (c[0]) {
    case 'e':
      mj_jiggler_enable = 1;
      mj_ledtiming_changed = true;
      break;
    case 'd':
      mj_jiggler_enable = 0;
      mj_ledtiming_changed = true;
      break;
    case 'r':
      mj_jiggler_enable = MJ_DEFAULT_JIGGLER_ENABLE;
      mj_ledtiming_changed = true;
      [[fallthrough]];
    case 's':
      Serial.print(F("Jiggler "));
      Serial.print(mj_jiggler_enable ? "En" : "Dis");
      Serial.print(F("abled\r\n"));
      break;
    case 'S':
      mj_eeprom_update_value(MJ_EEPROM_JIGGLER_ENABLE, 1, mj_jiggler_enable & 0x01);
      break;
    case 't':
      if (num >= 2) {
        switch (c[1]) {
          case '1':
          case '2':
            if (num >= 3) {
              value = (unsigned int) mj_convert_dec_long(c + 2);
              if (c[1] == '1') {
                if (value <= mj_jiggler_max) {
                  mj_jiggler_min = value;
                }
              } else {
                if (value >= mj_jiggler_min) {
                  mj_jiggler_max = value;
                }
              }
            }
            break;
          case 'r': // Restore Default Timings
            mj_jiggler_min = MJ_DEFAULT_JIGGLER_MIN;
            mj_jiggler_max = MJ_DEFAULT_JIGGLER_MAX;
            [[fallthrough]];
          case 's':
            Serial.print(F("Jiggler delay min: "));
            Serial.print(mj_jiggler_min);
            Serial.print(F(" max: "));
            Serial.print(mj_jiggler_max);
            Serial.print(F("\r\n"));
            break;
          case 'S':
            mj_eeprom_write_int(MJ_EEPROM_JIGGLER_MIN, mj_jiggler_min);
            mj_eeprom_update_value(MJ_EEPROM_JIGGLER_MAX, 2, mj_jiggler_max);
            break;
          default:
            break;
        }
      }
      break;
    case 'p': // Jiggler Pixel maximum moves
      if (num >= 2) {
        switch (c[1]) {
          case '0':
          case '1':
          case '2':
          case '3':
          case '4':
          case '5':
          case '6':
          case '7':
          case '8':
          case '9':
            value = (byte) mj_convert_dec_long(c + 1);
            mj_jiggler_pixel = value;
            break;
          case 'S':
            mj_eeprom_update_value(MJ_EEPROM_JIGGLER_PIXEL, 1, mj_jiggler_pixel);
            break;
          case 'r':
            mj_jiggler_pixel = MJ_DEFAULT_JIGGLER_PIXEL;
            [[fallthrough]];
          case 's':
            Serial.print(F("Jiggler Pixel max: "));
            Serial.print(mj_jiggler_pixel);
            Serial.print(F("\r\n"));
            break;
          default:
            break;
        }
      }
      break;
  }
}

void print_help(const char * textstart, int tabsize, const char * prefix) {
  int column = 0, firstspace = 0, plen, offset = 0, firstline = 1;
  char c;
  plen = strlen(prefix);
  while ((c = pgm_read_byte(textstart + offset)) > 0) {
    if (c == '\t') {
      firstspace = 1;
      if (column != 0) {
        column = 0;
        Serial.println("");
      }
      for (; column < tabsize; column++) {
        Serial.write(' ');
      }
    } else if (firstspace == 0 && c == ' ') {
      firstspace = 1;
      for (; column < (tabsize - 2); column++) {
        Serial.write(' ');
      }
      Serial.write('-');
      Serial.write(' ');
    } else if (c == '\n') {
      firstspace = 0;
      column = 0;
      Serial.println("");
    } else {
      if (column == 0) {
        Serial.print(prefix);
        column += plen;
        if (firstline == 1) {
          Serial.print(F("..."));
          column += 3;
          firstline = 0;
          firstspace = 1;
          for (; column < (tabsize - 3); column++) {
            Serial.write(' ');
          }
          Serial.print(F("-- "));
        }
      }
      Serial.write(c);
      column++;
    }
    offset++;
  }
}

// Simply prints the info message to serial console
void printinfo() {
  Serial.println(F(INFOSTRING));
}

void print_att_help() {
  static const char helptext[] PROGMEM = "AutoType Commands\n"
                                         "s show string in EEPROM\n"
                                         "k<string> type given string to keyboard\n"
                                         "w<string> set string in EEPROM"
                                         "\tfollowing Escape-Codes exist for the string:"
                                         "\t\\e -> ESC, \\n or \\r -> RETURN, \\t -> TAB, \\\\ -> \\,"
                                         "\t\\d<0-9> Set typing delay to 0-9, \\x<HH> Hex code <HH>,"
                                         "\t\\X<HH> same as \\x but without auto-remapping"
                                         "\t\\s<HH> \"Special Key\" no press/release - hold the key \\sff releases all keys\n"
                                         "t type string in EEPROM to keyboard\n"
                                         "d<0-9><number> sets delay <0-9> to <number> (0-4294967295) in ms\n"
                                         "ds show typing delays\n"
                                         "dS Save typing delays to EEPROM\n"
                                         "dr restore default typing delays\n"
                                         "at<number> set AutoType Delay to <number> (0-4294967295) ms"
                                         "\tWill automatically type the string defined with attw after given Delay to Keyboard"
                                         "\tThe Delay counts from Power-On of the Stick not of the PC! Be careful about this!\n"
                                         "ae enable AutoType - will only be active on reset so don't forget attaS\n"
                                         "ad disable AutoType\n"
                                         "as show AutoType Delay\n"
                                         "aS Save AutoType Delay to EEPROM\n"
                                         "ar restore default AutoType Delay\n";
  print_help(helptext, 20, "att");
}

// Writes Baud-rate of serial line as on a atmega32u4 the Serial.begin(9600) does not really set baud.
void parsebaud() {
  const char p[] = "NOEMS";
  const char s[][4] = { "1", "1.5", "2" };
  Serial.print(F("Baudrate is "));
  Serial.print(Serial.baud());
  Serial.print(F(" at "));
  Serial.print(Serial.numbits());
  Serial.print(F("/"));
  Serial.write(p[Serial.paritytype()]);
  Serial.print(F("/"));
  Serial.print(s[Serial.stopbits()]);
  Serial.print(F(" RTS: "));
  Serial.println(Serial.rts());
}

void print_at_help() {
  static const char helptext[] PROGMEM = "Possible Commands - type at<command>? for more help\n"
                                         "e EEPROM\n"
                                         "l LED\n"
                                         "i Version Info\n"
                                         "j MouseJiggler\n"
                                         "t AutoType\n"
                                         "b Input Buttons\n"
                                         "r Baud Rate\n"
                                         "?? Print full Help\n";
  print_help(helptext, 9, "at");
}

void print_atj_help() {
  static const char helptext[] PROGMEM = "Jiggler Commands\n"
                                         "e enable MouseJiggler\n"
                                         "d disable MouseJiggler\n"
                                         "s show Jiggler state\n"
                                         "S Save Jiggler state to EEPROM\n"
                                         "r restore default state\n"
                                         "p<number> set maximal pixel move 0-255\n"
                                         "pr restore default maximal pixel move\n"
                                         "ps show maximal pixel move\n"
                                         "pS Save maximal pixel move to EEPROM\n"
                                         "t1<number> set minimal time in ms between Jiggles\n"
                                         "t2<number> set maximal time in ms between Jiggles\n"
                                         "tr restore default minimal and maximal time\n"
                                         "ts show minimal and maximal time\n"
                                         "tS Save minimal and maximal time to EEPROM\n";
  print_help(helptext, 16, "atj");
}

void print_atl_help() {
  static const char helptext[] PROGMEM = "LED Commands\n"
                                         "p1<number> set LED1 pin to <number>\n"
                                         "p2<number> set LED2 pin to <number>\n"
                                         "ps show LED Pin assignments\n"
                                         "pS Save LED Pin assignments to EEPROM\n"
                                         "pr restore default LED Lin assignments"
                                         "\tyou cannot assign a pin that is already used by Buttons ot another LED\n"
                                         "t<1-4><0-65535) set timing modifier <1-4> to <0-65535>"
                                         "\t10000 is default, higher is slower"
                                         "\tnot all timings are used at every LED Mode\n"
                                         "ts show LED Timing modifier\n"
                                         "pS Save LED Timing mofifier to EEPROM\n"
                                         "pS restore default LED Timing mofifier\n"
                                         "m<0-65535> set LED Blink Mode (only 0-7 are defined yet)\n"
                                         "ms show LED Blink Mode\n"
                                         "mS Save LED Blink Mode to EEPROM\n"
                                         "mr restore default LED Blink Mode\n"
                                         "o<0-ffff> set LED Option. Only lowest 2 Bits are used atm. They invert LED1 or LED2 if set\n"
                                         "os show LED Option\n"
                                         "oS Save LED Option to EEPROM\n"
                                         "or restore default LED Option\n";
  print_help(helptext, 21, "atl");
}

void print_ate_help() {
  static const char helptext[] PROGMEM = "EEPROM Commands\n"
                                         "c Calculate CRC and show calculated and stored CRC\n"
                                         "C Calculate CRC and update stored CRC\n"
                                         "d dump all EEPROM Contents\n"
                                         "i initialize defaults to EEPROM. Needs to be given three times to activate\n"
                                         "I wipe and initialize defaults to EEPROM. Needs to be given three times to activate\n"
                                         "z wipe EEPROM. Only if you'd like to upload different Sketch. Needs to be given three times to activate\n"
                                         "wb<xxx><yy> write byte <yy> to EEPROM Address <xxx> - Data and Address in HEX\n"
                                         "wi<xxx><yyyy> write int <yyyy> to EEPROM Address <xxx> - Data and Address in HEX\n"
                                         "wl<xxx><y-yyyyyyyy> write long <y> to EEPROM Address <xxx> - Data and Address in HEX\n"
                                         "rb<xxx> - read byte from Address <xxx>\n"
                                         "ri<xxx> - read int from Address <xxx>\n"
                                         "rl<xxx> - read long from Address <xxx>\n";
  print_help(helptext, 25, "ate");
}

void print_atb_help() {
  static const char helptext[] PROGMEM = "Button Assignment Commands\n"
                                         "<1-4><number> sets button <1-4> to pin <number>"
                                         "\tyou cannot assign a pin that is already used by LED or another button\n"
                                         "s shows button to pin assignments\n"
                                         "S Save button to pin assignments to EEPROM\n"
                                         "r restore default button to pin assignments\n";
  print_help(helptext, 19, "atb");
}

void print_atr_help() {
  static const char helptext[] PROGMEM = "Baudrate Command\n"
                                         " shows the baud-Rate the PC is connected with\n";
  print_help(helptext, 9, "atb");
}

// Parse string given via serial and pass thru different decoders
void parsestring(char *c, unsigned int num) {
  if (num > 2) {
    if (c[0] == 'a' && c[1] == 't') {  // String needs to start with 'at'
      if (num == 4) {
        if (c[3] == '?') {
          switch (c[2]) {
            case 'j':
              printinfo();
              print_atj_help();
              break;
            case 'l':
              printinfo();
              print_atl_help();
              break;
            case 'i':
              printinfo();
              print_at_help();
              break;
            case 'e':
              printinfo();
              print_ate_help();
              break;
            case 't':
              printinfo();
              print_att_help();
              break;
            case 'b':  // input buttons
              printinfo();
              print_atb_help();
              break;
            case 'r':  // print actual baud rate
              printinfo();
              print_atr_help();
              break;
            case '?':  // print actual baud rate
              printinfo();
              print_at_help();
              Serial.print(F("\r\n"));
              print_atj_help();
              Serial.print(F("\r\n"));
              print_atl_help();
              Serial.print(F("\r\n"));
              print_ate_help();
              Serial.print(F("\r\n"));
              print_att_help();
              Serial.print(F("\r\n"));
              print_atb_help();
              Serial.print(F("\r\n"));
              print_atr_help();
              break;
            default:
              printinfo();
              print_at_help();
              break;
          }
        }
      }
      switch (c[2]) {  // The following character decides which submodule will decode further
        case 'j':  // JIGGLER related
          parsejiggler(c + 3, num - 3);
          break;
        case 'l':  // LED related
          parseled(c + 3, num - 3);
          break;
        case 'i':  // Info Message
          printinfo();
          break;
        case 'e':  // EEPROM related
          parseeeprom(c + 3, num - 3);
          break;
        case 't':  // autotype related
          parsekeyboard(c + 3, num - 3);
          break;
        case 'b':  // input buttons
          parsebuttons(c + 3, num - 3);
          break;
        case 'r':  // print actual baud rate
          parsebaud();
          break;
        default:
          if (num == 3) {
            print_at_help();
          }
          break;
      }
    }
  } else {
    print_at_help();
  }
}
